/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Path planning and obstacle avoidance algorithm
Author : Sharayu Ghangrekar
Last updated :  5th April 2009    

This version of software handles:
 - Navigation of ATV in the field without any obstacle
 - Obstacle avoidance by going from left or right hand side depending on whichever local path have   
   least number of intermediate points
 - Obstacle can be present anywhere including on boundries
 - this version optimizes the navigation 
   by not visting the points already visited (which may have been visited during local path)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <conio.h>

//Global Variables

int ScanPtNumber;
int NoOfXScanPoints;      
int NoOfYScanPoints;      
int FieldWidth, VehicleWidth, FieldLength, LidarRange;
int XScanRange;
int YScanRange;
int TotalNoOfScanPts;
int Xcord, Ycord;
int NumOfKownObst;
int LocalNavCounter;
int LocalPathScanPtArray[10000];
int LocalLeftPathScanPtArray[10000];
int LocalRightPathScanPtArray[10000];
int TemporaryScanPtAccess[10000];
int Unknown_X_Arr[65000];
int Unknown_Y_Arr[65000];
int mainSource,mainPrev, mainTarget;
int stackCounter;
int LocalPathPrevScanPt;
int iTempLocalNavCounter=0; 
int LocalPathCounter=0;
int NumOfUnKownObst = 0; 
int UnkownImagSqr_Left_X;
int UnkownImagSqr_Left_Y;
int UnkownImagSqr_Right_X;
int UnkownImagSqr_Right_Y;
int UnkownImagSqr_Top_X;
int UnkownImagSqr_Top_Y;
int UnkownImagSqr_Bottom_X;
int UnkownImagSqr_Bottom_Y; 
int OffPathDirection;
int counterISqr;

int iPrevScanPt;  
int iCurrPosScanPt = 1;
int iLocalPath = 0;
int TemporaryScanPtAccessCtr = 0;
int FieldCompletlyCovered = 0; //This variable indicates if Field is completely covered
int UnknownObstacleFound = 0;



struct Coordinates
{
    int X;
    int Y;
}Curr, Next, Prev;

struct Obst
{
     Coordinates North, South, East, West;
     Coordinates ImgSqrNE, ImgSqrNW, ImgSqrSE, ImgSqrSW;
     int north, south, east, west;
}KnownObstacle[100], UnKnownObstacle[100];

struct ScanPt
{
    Coordinates Source, Goal, Igoal;
    int ScanPtNumsOfReachPts[4];       // stores scan point number of each of reach points
    int ScanPtNum;                      // number to identify a scan point
    int GoalPtNum;                      // number to identify Goal of current scan point
    int ReachPtAccessibility[10][10];   // identifies accessibility of each reach point
    int NumOfReachPts;                  // total number of reach points for a particular scan point
    int Visited;                       // indicates of the scan point is already visited
    int IsScanPointReachable;          // indicates if the scan point can be reached
    int ObstacleNo;                     // identifies the obstacle which makes this point unreachbale
    int AltGoalPtNum;                   // alternate goal pt in case of known obstacles
    int Weightage;                      //Weightage for every ScanPoint. The ones visted more times get more Weightage
} ScanPtArr[10000],SimulationScanPtArr[10000];

struct LocalPathInfo
{
       int ScanPtNum;
       int Direction;
} stackScanPtArray[10000];
#define STRIAGHT 1
#define LEFT     2
#define RIGHT    3
#define REVERSE  4

#define LEFTDIR 0
#define RIGHTDIR 1
#define TOPDIR 2
#define BOTTOMDIR 3
#define NULLDIR 4
     
     
int VehicleNavigationArr[1000];
int VehicleNavCtr = 0;
int VehicleNavTempCtr = 1;

FILE *fpOut;				//Declares file pointer fpOut of type FILE for Output

//Declare Functions           
void VarInit(void);           
void CreateScanGoalPts(void);     
void CreateReachPts(void);
void NavigateField(void);
int  ObstacleDetection(int currScanPtNum, int goalScanPtNum);
void MoveVehicle(int prevScanPtNum, int currScanPtNum, int goalScanPtNum);
void CreateImgSqr(void);
void ScanPtInsideImSqr(void);
void CreateAltGoalPt(void);
void LocalPathNav(int prevScanPtNum, int sourceScanPtNum, int goalScanPtNum, int Pattern);
int FindOptimumLocalGoalPt(int SourcePt, int AltGoalPt);
int FindRightOptimumLocalGoalPt(int SourcePt, int AltGoalPt);
int GetScanPtNum(int X, int Y);
void ReadInputLineFromFile(void);
void UnKnownImagSqrDetection(void);
void LocalPathNavReInitialize(void);     
void CreateImgSqrForUnkownObst(void);
void ScanPtInsideUnkownImSqr(void);
void ScanPointReachabilityCheck(void);
void swap(int *x,int *y);
void bublesort(int list1[],int n);
int CheckIfTemporaryInAccessible(int ScanPt);
void OptimumPathDecision(void);
void ArrayCopy(int list1[],int list2[],int n);
void UpdateAltGoalPt(int iCurrPos);

//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

int XOffSet = 100;
int YOffSet = 100;
char sVar[10];//used for temporary integer to string conversion

static char g_szClassName[] = "MyWindowClass";
static HINSTANCE g_hInst = NULL;

const UINT idTimer1 = 1;
UINT nTimerDelay = 800;


HBITMAP hbmCar, hbmCarMask;
BITMAP bm;

HDC hdc;
int CarX, CarY;
int localNav,local1;
int Redraw;

int deltaValue = 4;

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          EraseCar
Input Parameters: Handle to windows
Output Parameters: Null
Description:       Once the car moves to the next location, the image on its last position is
                   erased. In this simulation it is painted in the color same as the field.
Last updates:      10th Feb 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void EraseCar(HDC hdc)
{
   RECT rc;
   rc.left = CarX;
   rc.top = CarY;
   rc.right = CarX + bm.bmWidth;
   rc.bottom = CarY + bm.bmHeight;
 
   FillRect(hdc, &rc, CreateSolidBrush(RGB(130, 130, 230))); // RGB color set to be same as the field color


}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          DrawCar
Input Parameters: Handle to windows
Output Parameters: Null
Description:       considers the Cars Bitmap Image from the memory. Ass per its size and memory
                   paints the car and its mask on the windows
Last updates:      10th Feb 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void DrawCar(HDC hdc)
{
   HDC hdcMemory;
   hdcMemory = CreateCompatibleDC(hdc);
   
/*
Before selecting the newly created bitmap object, allocate a block of computer memory that
would hold the bitmap. YIt can be then copied to the actual device. This job is taken care of
by the CreateCompatibleDC() function.
*/   

   SelectObject(hdcMemory, hbmCarMask);
   BitBlt(hdc, CarX, CarY, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCAND);

   SelectObject(hdcMemory, hbmCar);
   BitBlt(hdc, CarX, CarY, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCPAINT);

   DeleteDC(hdcMemory);
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          UpdateCar
Input Parameters: Handle to windows
Output Parameters: Null
Description:       Updates the Car position seen on the windows for simulation as per 
                   the algorithm. Also the basic 'move vehicle' function is called 
                   here to coordinate the functioniing of the algorithm and the simulation
Last updates:      10th Feb 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void UpdateCar(HWND hwnd)
{
     int iCounter;
     int iDirection;

   RECT rc;
   GetClientRect(hwnd, &rc);
 

    if (iLocalPath==1)
    {
       if (iTempLocalNavCounter >=2)
       {
          MoveVehicle(ScanPtArr[LocalPathScanPtArray[iTempLocalNavCounter]].ScanPtNum, ScanPtArr[LocalPathScanPtArray[iTempLocalNavCounter-1]].ScanPtNum, ScanPtArr[LocalPathScanPtArray[iTempLocalNavCounter-2]].ScanPtNum);                                
          iTempLocalNavCounter--;
       }
       else
           {
            iLocalPath = 0;
            
            LocalPathNavReInitialize();
            }
    }
    else if ((iCurrPosScanPt != TotalNoOfScanPts) && (FieldCompletlyCovered!=1))
       // Totalnoofscanpt -1 done so that loop goes till second last point and in the final loop vehicle 
        //is moved to final destination
     {

               //During normal navigation PrevSacPt is CurrScanPt-1 of the ScanPtArr array
                iPrevScanPt = ScanPtArr[iCurrPosScanPt-1].ScanPtNum;
                
                if(ScanPtArr[iCurrPosScanPt].GoalPtNum == ScanPtArr[iCurrPosScanPt].AltGoalPtNum)
                
                {
                      //Following If bloock checks if LocalPath was taken before this point and modifies PrevScanPt
                      if (LocalPathPrevScanPt !=0)
                      {
                           iPrevScanPt = LocalPathPrevScanPt;  
                           LocalPathPrevScanPt = 0;               
                      }
                      MoveVehicle(iPrevScanPt, ScanPtArr[iCurrPosScanPt].ScanPtNum, ScanPtArr[iCurrPosScanPt].GoalPtNum);
                                                           
                }
                else
                {
                     // start local path navigation in case of imaginary square
                     // "AltGoalPtNum" is send for this navigation
                     // moves vehicle around the obstacle from source point to the alternate goal point
                     iLocalPath = 1;
                     iDirection = 0;
                     OptimumPathDecision();
                     
                     //LocalPathNav(iPrevScanPt, iCurrPosScanPt, ScanPtArr[iCurrPosScanPt].AltGoalPtNum, iDirection); 
                     //iTempLocalNavCounter = stackCounter;
                     iTempLocalNavCounter = LocalPathCounter;

               }

   VehicleNavTempCtr++;         
   }//end-if Vehicle...
}//end WINPROC

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          WndProc
Input Parameters:  Handle to windows, message number, 32 bit word and long parameter
Output Parameters: 32 bittinteger
Description:       Function pointer to manage the message sent
Last updates:      10th Feb 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
      HBRUSH NewBrush;
      HBRUSH NewObstBrush;
      HPEN hPen;
      HDC hDC;
    COLORREF clrFieldTitle = RGB(100, 0, 200);// purple
    COLORREF clrTxtBckgrnd = RGB(255, 255, 0); // yellow
    COLORREF clrScnPtTxtBkgrd = RGB(130, 130, 230); // green
    local1 = 0;
    

   switch(Message)
   {
        
      case WM_CREATE:
          hbmCar = LoadBitmap(g_hInst, "CARBMP");
         hbmCarMask = LoadBitmap(g_hInst, "CARMASKBMP");
         if(!hbmCar || !hbmCarMask)
         {
            MessageBox(hwnd, "Load of resources failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
         }

         GetObject(hbmCar, sizeof(bm), &bm);
         SetTimer(hwnd, idTimer1, nTimerDelay, NULL);

         VarInit();                 // Initialize variables            
         CreateScanGoalPts();       // Create scan point numbers, their X,Ycoords and their goal points
         CreateReachPts();          // Create reach points for each scan point
         ScanPtInsideImSqr();
         CreateAltGoalPt();            // update goal points as per known obstacles

                               

         CarX = XOffSet;
         CarY = YOffSet;// navigation starting from topleft
         

      break;
      case WM_TIMER:
         if(hbmCar && hbmCarMask)
         {
            HDC hdcWindow;
            hdcWindow = GetDC(hwnd);
            int iCounter, iTemp;
            char ScanPtNo;
            
                      
            EraseCar(hdcWindow);
            UpdateCar(hwnd);
            DrawCar(hdcWindow);
            
            ReleaseDC(hwnd, hdcWindow);
         }
      break;
      case WM_PAINT:

            PAINTSTRUCT ps;
            HDC hdcWindow;            
            hdcWindow = BeginPaint(hwnd, &ps);

            //Function calls for Path Planning
            char c;
            int temp;
            int iCounter;
            int Ctr;
            int ictr;
            char ScanPtNo;
            int iTextPosXOffSet;
            
            DrawCar(hdcWindow);
//---------------------------------------------------------------------------            

//HDC BeginPaint(HWND hWnd, LPPAINTSTRUCT lpPaint);        
        NewBrush = CreateSolidBrush(RGB(130, 130, 230));//fill in faint blue color for the field
//COLORREF RGB(BYTE byRed, BYTE byGreen, BYTE byBlue); 
//--- 8 bytes each for red,green and blue color
        SelectObject(hdcWindow, NewBrush); 
//HGDIOBJ SelectObject(HDC hdc, HGDIOBJ hgdiobj);
        Rectangle(hdcWindow, XOffSet, YOffSet, (XOffSet+(int)FieldWidth), (YOffSet+(int)FieldLength));
        //Rectangle(hdcWindow, XOffSet, YOffSet, 550, 550);
//BOOL Rectangle(HDC hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect);        
       DeleteObject(NewBrush); // end of green brush color used only for the field rectangle
       

              // draw imaginary square around all known obtsales       
       for(temp = 0; temp<NumOfKownObst;temp++) // for all known obstacles
        {
                // draw imaginary square around the obstacle
                Rectangle(hdcWindow, KnownObstacle[temp].ImgSqrNW.X, KnownObstacle[temp].ImgSqrNW.Y, KnownObstacle[temp].ImgSqrNE.X, KnownObstacle[temp].ImgSqrSW.Y);
            
        }

//Draw scan point// end of green brush color to be used only for the field rectangle[-s - small circles at all the locations where scan points are present
       for(temp = 1; temp <=TotalNoOfScanPts ; temp++)//TotalNoOfScanPts
       {
                
                if(SimulationScanPtArr[temp].IsScanPointReachable == 0)
                {
                       NewBrush = CreateSolidBrush(RGB(0,0,250)); // new yellow brush color for reachable scan pts
                       SelectObject(hdcWindow, NewBrush);    
                       Ellipse(hdcWindow, (SimulationScanPtArr[temp].Source.X-4.0), (SimulationScanPtArr[temp].Source.Y-4.0), (SimulationScanPtArr[temp].Source.X+4.0), (SimulationScanPtArr[temp].Source.Y+4.0));
                       DeleteObject(NewBrush);// end of purple brush color used only for the scan pt circles                       
                }
                else
                {
                       NewBrush = CreateSolidBrush(RGB(255, 180, 20)); // new purple brush color for unreachable scan pts
                       SelectObject(hdcWindow, NewBrush);    
                       Ellipse(hdcWindow, (SimulationScanPtArr[temp].Source.X-4.0), (SimulationScanPtArr[temp].Source.Y-4.0), (SimulationScanPtArr[temp].Source.X+4.0), (SimulationScanPtArr[temp].Source.Y+4.0));
                       DeleteObject(NewBrush);// end of purple brush color used only for the scan pt circles
                }
                
      
                    //BOOL Ellipse(HDC hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect);         
           ScanPtNo = (char)temp;
           SetBkColor(hdcWindow, clrScnPtTxtBkgrd);
           //following line converts integer ScanPtNum into string and stores in variable sVar       
           sprintf( sVar, "%d", ScanPtArr[temp].ScanPtNum);
           iTextPosXOffSet = 30;
           if  (strlen(sVar)>2)
               iTextPosXOffSet = 2;
           TextOut(hdcWindow, (SimulationScanPtArr[temp].Source.X-9), (SimulationScanPtArr[temp].Source.Y-30.0), sVar, strlen(sVar));
       }//end-for(temp = 1..
     
//-----------------KNOWN OBSTACLE---------------------------
         NewObstBrush = CreateSolidBrush(RGB(90, 190, 90));
         SelectObject(hdcWindow, NewObstBrush);
         for (iCounter = 0; iCounter < NumOfKownObst; iCounter++)
         {
             Ellipse(hdcWindow, (int)KnownObstacle[iCounter].west, (int)KnownObstacle[iCounter].north, (int)KnownObstacle[iCounter].east, (int)KnownObstacle[iCounter].south);
         }
         DeleteObject(NewBrush);
              
        SetBkColor(hdcWindow, clrTxtBckgrnd);
        SetTextColor(hdcWindow, clrFieldTitle); 
        TextOut(hdcWindow, 480, 10, "Path Planning Field", 19);

          EndPaint(hwnd, &ps);        
        
      break;
      
        case WM_LBUTTONDOWN:
            hdc = GetDC(hwnd);
            if(hdc)
            {
              short xp = LOWORD(lParam);
              short yp = HIWORD(lParam);	
	
              UnknownObstacleFound = 0;
              MoveToEx(hdc, xp, yp, NULL);
              
              
// first point of contact X and Y stored as first elements of the array
              Unknown_X_Arr[0] = xp; 
              Unknown_Y_Arr[0] = yp;
              
           }
            break;
            
  case WM_LBUTTONUP:
    if(hdc)
    {    
    NumOfUnKownObst++;     
    UnknownObstacleFound = 1;


   //sort both the arrays in assendin order of X and Y coordinate    
bublesort(Unknown_X_Arr,counterISqr);
bublesort(Unknown_Y_Arr,counterISqr);
UnkownImagSqr_Right_X = Unknown_X_Arr[counterISqr-1] + (VehicleWidth/4);
UnkownImagSqr_Top_Y = Unknown_Y_Arr[0] - (VehicleWidth/4);
UnkownImagSqr_Left_X = Unknown_X_Arr[0] - (VehicleWidth/4);
UnkownImagSqr_Bottom_Y = Unknown_Y_Arr[counterISqr-1] + (VehicleWidth/4);    
    

    CreateImgSqrForUnkownObst();
    ScanPtInsideUnkownImSqr();
    CreateAltGoalPt();
    ReleaseDC(hwnd, hdc);

/*
The X of start and end points are considered for left and right and
the Y of start and end points are considered for top and bottom respectively

Hence to draw the unknown obstacle -- RESTRICTION ::
      THE UNKNWON OBSTACLE LINE / OBJECT CAN BE DRAWN ONLY FROM LEFT TO RIGHT AND TOP TO 
      BOTTOM DIRECTION !!
*/

             //clear the arrays for next unknown obstacle         
             for(temp = 0; temp<=counterISqr;temp++)         
             {
                  Unknown_X_Arr[temp]  = 0;
                  Unknown_Y_Arr[temp]  = 0;   
             }
             counterISqr = 1;


      hdc = NULL;
    }
    break;
  case WM_MOUSEMOVE:
    if(hdc)
    {
      short xp = LOWORD(lParam);
      short yp = HIWORD(lParam);
      LineTo(hdc, xp, yp);
      
      
      // first point of contact X and Y stored as first elements of the array
              Unknown_X_Arr[counterISqr] = xp; 
              Unknown_Y_Arr[counterISqr] = yp;
              counterISqr++;

    }
    break;
    
      case WM_CLOSE:
         DestroyWindow(hwnd);
      break;
      case WM_DESTROY:
         KillTimer(hwnd, idTimer1);
         
         DeleteObject(hbmCar);
         DeleteObject(hbmCarMask);
         PostQuitMessage(0);
      break;
      default:
         return DefWindowProc(hwnd, Message, wParam, lParam);
   }
   return 0;
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          WinMain
Input Parameters:  Handle to the windows application
Output Parameters: integer
Description:       Main function for Win API which defines the windows parameters
Last updates:      10th Feb 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpCmdLine, int nCmdShow)
{
   WNDCLASSEX WndClass;
   HWND hwnd;
   MSG Msg;
   
   g_hInst = hInstance;

   WndClass.cbSize        = sizeof(WNDCLASSEX);
   WndClass.style         = 0;
   WndClass.lpfnWndProc   = WndProc;
   WndClass.cbClsExtra    = 0;
   WndClass.cbWndExtra    = 0;
   WndClass.hInstance     = g_hInst;
   WndClass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
   WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
   WndClass.hbrBackground = (HBRUSH)(COLOR_BTNFACE+4); // backgrouns color of the window
   WndClass.lpszMenuName  = NULL;
   WndClass.lpszClassName = g_szClassName;
   WndClass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

   if(!RegisterClassEx(&WndClass))
   {
      MessageBox(0, "Window Registration Failed!", "Error!",
         MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
      return 0;
   }

   hwnd = CreateWindowEx(
      WS_EX_CLIENTEDGE,
      g_szClassName,
      "A Bitmap Program",
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT, 900, 700,//window dimensions
      NULL, NULL, g_hInst, NULL);

   if(hwnd == NULL)
   {
      MessageBox(0, "Window Creation Failed!", "Error!",
         MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
      return 0;
   }

   ShowWindow(hwnd, nCmdShow);
   UpdateWindow(hwnd);

   while(GetMessage(&Msg, NULL, 0, 0))
   {
      TranslateMessage(&Msg);
      DispatchMessage(&Msg);
   }
   return Msg.wParam;
}


/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          CreateScanGoalPts
Input Parameters:  Null
Output Parameters: Null
Description:       For all scan points, assigns specific number, calculates the X and Y coordinates and their goal points 
Last updates:      7th Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void CreateScanGoalPts(void)
{
     int iCountX, iCountY; 
     int iCounter, temp;
    
     iCounter = 0;
     NoOfXScanPoints = (int)(ceil(FieldWidth/VehicleWidth))+1;
    NoOfYScanPoints = (int)(ceil(FieldLength / (LidarRange/2)))+1;
    XScanRange = FieldWidth / (NoOfXScanPoints - 1);
    YScanRange = FieldLength / (NoOfYScanPoints-1);
    TotalNoOfScanPts = NoOfXScanPoints * NoOfYScanPoints;
    
    
    //calculate scan point number and corresponding X and Y coordinates for each coordinate
    for(iCountX=0; iCountX < NoOfXScanPoints; iCountX ++)
    {
        for(iCountY=0; iCountY < NoOfYScanPoints; iCountY ++)
        {
            Xcord = (iCountX * XScanRange);  
            Ycord = (iCountY * YScanRange);
            
            if((iCountX % 2) != 0) //if iCountX is odd
            {
                ScanPtNumber = (iCounter - (iCountY*2))+ NoOfYScanPoints;
            }
            else
                ScanPtNumber = iCounter +1 ;
            ScanPtArr[ScanPtNumber].ScanPtNum = ScanPtNumber;
            ScanPtArr[ScanPtNumber].Source.X = Xcord;
            ScanPtArr[ScanPtNumber].Source.Y = Ycord;
            iCounter++;
            
        }//for iCountY
    }//for iCountX
    
    //for calculating Goal points -- goal point number = scan point number + 1
    for(temp=1; temp < TotalNoOfScanPts; temp ++)
    {
         ScanPtArr[temp].GoalPtNum = ScanPtArr[temp].ScanPtNum + 1;
         ScanPtArr[temp].AltGoalPtNum = ScanPtArr[temp].GoalPtNum; // by default alternate goal pt is same as normal goal pt
         ScanPtArr[temp].Goal.X = ScanPtArr[(temp+1)].Source.X ;
         ScanPtArr[temp].Goal.Y = ScanPtArr[(temp+1)].Source.Y ;
         ScanPtArr[temp].IsScanPointReachable = 1; // default all scan points are considered reachable initiallly
         ScanPtArr[temp].Visited = 0; // default all scan points are considered non visited initially
         ScanPtArr[temp].Weightage = 1;

    }  
    
    for(temp=1; temp <= TotalNoOfScanPts; temp ++)
    {
       SimulationScanPtArr[temp].Source.X = ScanPtArr[temp].Source.X + XOffSet;   
       SimulationScanPtArr[temp].Source.Y = ScanPtArr[temp].Source.Y + YOffSet;   
       SimulationScanPtArr[temp].GoalPtNum = ScanPtArr[temp].GoalPtNum;
       SimulationScanPtArr[temp].AltGoalPtNum = ScanPtArr[temp].AltGoalPtNum;
       SimulationScanPtArr[temp].ScanPtNum = ScanPtArr[ScanPtNumber].ScanPtNum ;
         SimulationScanPtArr[temp].IsScanPointReachable = 1; // default all scan points are considered reachable initiallly
         SimulationScanPtArr[temp].Visited = 0; // default all scan points are considered non visited initially
         SimulationScanPtArr[temp].Weightage = 1;
           
    }
    
       ScanPtArr[TotalNoOfScanPts].IsScanPointReachable = 1; // default all scan points are considered reachable initiallly
       ScanPtArr[TotalNoOfScanPts].Visited = 0; // default all scan points are considered non visited initially
       ScanPtArr[TotalNoOfScanPts].Weightage = 1;
       SimulationScanPtArr[TotalNoOfScanPts].IsScanPointReachable = 1; // default all scan points are considered reachable initiallly
       SimulationScanPtArr[TotalNoOfScanPts].Visited = 0; // default all scan points are considered non visited initially

}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          VarInit
Input Parameters:  Null
Output Parameters: Null
Description:       Initializes various variables required for the main function
Last updates:      7th Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void VarInit(void)
{

    ScanPtNumber = 1;
    LocalPathPrevScanPt = 0;
    counterISqr = 1;
    OffPathDirection = NULLDIR;  
    ReadInputLineFromFile();

     fpOut = fopen("Output Path Planning.txt", "w"); //opens NetList.txt in read mode. File pointer fp points to this file.
     //Check if file specified can be opened or not. If file can not be opened, exit ytthis function with return value 0.
     if (fpOut==NULL) 
     {
        printf("File Output.txt can not be opened\n");
        return;
     }
    
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          CreateScanGoalPts
Input Parameters:  Null
Output Parameters: Null
Description:       Assigns appropriate reach points for each scan points 
Last updates:      10th Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void CreateReachPts(void)
{
     int temp,temp1;
     int NumOfReachPoints;

     
     for(temp=1;temp<=TotalNoOfScanPts;temp++)
     {
                NumOfReachPoints = 0;
                //considre reach pt at left
                Xcord = ScanPtArr[temp].Source.X - XScanRange;
                Ycord = ScanPtArr[temp].Source.Y;
                if (Xcord >=0)
                {
                   ScanPtArr[temp].ScanPtNumsOfReachPts[0] = GetScanPtNum(Xcord,Ycord);
                   NumOfReachPoints ++;
                }  
                else
                    ScanPtArr[temp].ScanPtNumsOfReachPts[0] = -99;

                //consider ReachPt at Right
                Xcord = ScanPtArr[temp].Source.X + XScanRange;
                Ycord = ScanPtArr[temp].Source.Y;
                if (Xcord <=FieldWidth)
                {
                   ScanPtArr[temp].ScanPtNumsOfReachPts[1] = GetScanPtNum(Xcord,Ycord);
                   NumOfReachPoints ++;
                }
                else
                    ScanPtArr[temp].ScanPtNumsOfReachPts[1] = -99;

                //consider ReachPt at top
                Xcord = ScanPtArr[temp].Source.X;
                Ycord = ScanPtArr[temp].Source.Y - YScanRange;
                if (Ycord >=0)
                {
                   ScanPtArr[temp].ScanPtNumsOfReachPts[2] = GetScanPtNum(Xcord,Ycord);
                   NumOfReachPoints ++;
                }
                else
                    ScanPtArr[temp].ScanPtNumsOfReachPts[2] = -99;

                //consider ReachPt at bottom
                Xcord = ScanPtArr[temp].Source.X;
                Ycord = ScanPtArr[temp].Source.Y + YScanRange;
                if (Ycord <= FieldLength)
                {
                   ScanPtArr[temp].ScanPtNumsOfReachPts[3] = GetScanPtNum(Xcord,Ycord);
                   NumOfReachPoints ++;
                }
                else
                    ScanPtArr[temp].ScanPtNumsOfReachPts[3] = -99;

            ScanPtArr[temp].NumOfReachPts = NumOfReachPoints ;    
           //fprintf(fpOut, "\nScanPoint = %d\nReachPoint[0] = %d\nReachPoint[1] = %d\nReachPoint[2] = %d\nReachPoint[3] = %d\n\n", temp, ScanPtArr[temp].ScanPtNumsOfReachPts[0],ScanPtArr[temp].ScanPtNumsOfReachPts[1],ScanPtArr[temp].ScanPtNumsOfReachPts[2],ScanPtArr[temp].ScanPtNumsOfReachPts[3]);
     }//end-for
    
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          NavigateField
Input Parameters:  Null
Output Parameters: Null
Description:       Starts navigating through the field.For every scan point in the field, it checks 
                   for presence of obstacle. If so then the navigates through a local path.
                   Else moves the vehicle along its normal path.
Last updates:      11th Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void NavigateField(void)
{
     int iCounter;
     int ObstPresent;
     int iPrevScanPt;
     char c;
     
     for(iCounter=1;iCounter <= (TotalNoOfScanPts-1); iCounter ++)   
        // Totalnoofscanpt -1 done so that loop goes till second last point and in the final loop vehicle 
        //is moved to final destination
     {

            if(ScanPtArr[iCounter].IsScanPointReachable == 0)
              continue;
            c = getchar();
            //During normal navigation PrevSacPt is CurrScanPt-1 of the ScanPtArr array
            iPrevScanPt = ScanPtArr[iCounter].ScanPtNum-1;
            
            if(ScanPtArr[ScanPtArr[iCounter].GoalPtNum].IsScanPointReachable == 1)//no known obstacle/imaginary square in front of current scan point
            {
                  //Following If bloock checks if LocalPath was taken before this point and modifies PrevScanPt
                  if (LocalPathPrevScanPt !=0)
                  {
                       iPrevScanPt = LocalPathPrevScanPt;  
                       LocalPathPrevScanPt = 0;               
                  }
                  MoveVehicle(iPrevScanPt, ScanPtArr[iCounter].ScanPtNum, ScanPtArr[iCounter].GoalPtNum);                                      
            }
            else if(ScanPtArr[ScanPtArr[iCounter].GoalPtNum].IsScanPointReachable == 0) //known obstacle/imaginary square present in front of current scan point
            {
                 // start local path navigation in case of imaginary square
                 // "AltGoalPtNum" is send for this navigation
                 // moves vehicle around the obstacle from source point to the alternate goal point
                 //LocalPathNav(ScanPtArr[iCounter].ScanPtNum-1, ScanPtArr[iCounter].ScanPtNum, ScanPtArr[iCounter].AltGoalPtNum); 
            }


      }//for
      VehicleNavigationArr[VehicleNavCtr++] = TotalNoOfScanPts;
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          ObstacleDetection
Input Parameters:  Scan point numbers of Current and its goal scan point
Output Parameters: 1 - If obstacle exists between current and its goal point
                   0 - If obstacle does not exists between current and its goal point
Description:       Gets input from LIDAR and calculates obstacle position with respect to scan points
Last updates:      10th Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
int ObstacleDetection(int currScanPtNum, int goalScanPtNum)
{
    //currScanPtNum and goalScanPtNum -- to be used by LIDAR routine to give this output
    char str[2];
    char c;
    
    printf("\n Enter 'Y' if obstacle present and 'N' if absent between scan point %d and its goal point %d ",currScanPtNum,goalScanPtNum);
     gets(str);
    if(str[0]=='Y')
    {
            return(1);
    }
    if(str[0]=='N')
    {
            return(0);
    }
  //  fflush(stdout);
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          MoveVehicle
Input Parameters:  X and Y coordinates of previous, current,and its goal next scan point
Output Parameters: Null
Description:       Based on coordinates of previous, current and goal scan point this function decides
                   the direction (left,right or straight) to be taken so as to reach the goal point.
                   This function calls mechanical move function to physically move the vehicle.
Last updates:      11th Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void MoveVehicle(int prevScanPtNum, int currScanPtNum, int goalScanPtNum)
{
     int val;
     int dFactor;
     int direction;

     VehicleNavigationArr[VehicleNavCtr++] = currScanPtNum;

     
    if(ScanPtArr[goalScanPtNum].Source.X == ScanPtArr[prevScanPtNum].Source.X)
        direction = STRIAGHT;
        
    if(ScanPtArr[goalScanPtNum].Source.Y == ScanPtArr[prevScanPtNum].Source.Y)
        direction = STRIAGHT;
    else
    {
        if((ScanPtArr[prevScanPtNum].Source.X == ScanPtArr[currScanPtNum].Source.X)&&
           (ScanPtArr[currScanPtNum].Source.X != ScanPtArr[goalScanPtNum].Source.X))
        {
            dFactor = ((ScanPtArr[goalScanPtNum].Source.X-ScanPtArr[currScanPtNum].Source.X)*
                       (ScanPtArr[currScanPtNum].Source.Y-ScanPtArr[prevScanPtNum].Source.Y));
            if (dFactor > 0)
            direction = 2;        // left
            if (dFactor < 0)
            direction = 3;        // right
        }
        if ((ScanPtArr[prevScanPtNum].Source.Y == ScanPtArr[currScanPtNum].Source.Y)&&
            (ScanPtArr[currScanPtNum].Source.Y != ScanPtArr[goalScanPtNum].Source.Y)) 
        {
            dFactor = ((ScanPtArr[goalScanPtNum].Source.Y-ScanPtArr[currScanPtNum].Source.Y)*
                       (ScanPtArr[currScanPtNum].Source.X-ScanPtArr[prevScanPtNum].Source.X));
            if (dFactor < 0)
            direction = 2;        // left
            if (dFactor > 0)
            direction = 3;        // right
        }
    }
        
    if (prevScanPtNum == goalScanPtNum)
         direction = REVERSE;
    
    ScanPtArr[goalScanPtNum].Visited = 1; // once vehicle is moved to the goal pt, it is markde as visited

    
        switch(direction)
        {
           case 1:
           fprintf(fpOut,"\n Vechicle moved to scan point %d by going in STRIAGHT direction\n",goalScanPtNum);
           printf("\n Vechicle moved to scan point %d by going in STRIAGHT direction",goalScanPtNum);
           break; 
           case 2:
           fprintf(fpOut,"\n Vechicle moved to scan point %d by going in LEFT direction\n",goalScanPtNum);
           printf("\n Vechicle moved to scan point %d by going in LEFT direction",goalScanPtNum);
           break;
           case 3:
           fprintf(fpOut,"\n Vechicle moved to scan point %d by going in RIGHT direction\n",goalScanPtNum);
           printf("\n Vechicle moved to scan point %d by going in RIGHT direction",goalScanPtNum);           
           break;     
           case 4:
           fprintf(fpOut,"\n Vechicle moved to scan point %d by turning back 180 degrees\n",goalScanPtNum);
           printf("\n Vechicle moved to scan point %d by turning back 180 degrees",goalScanPtNum);           
           break;     
           default:
           fprintf(fpOut,"\n Vechicle moved to scan point %d by going in STRIAGHT direction\n",goalScanPtNum);
           printf("\n Vechicle moved to scan point %d by going in STRIAGHT direction",goalScanPtNum);  
           break;     
        }
       CarX = SimulationScanPtArr[goalScanPtNum].Source.X-15;
       CarY = SimulationScanPtArr[goalScanPtNum].Source.Y-15;

           
       iCurrPosScanPt =  goalScanPtNum; 
       UpdateAltGoalPt(iCurrPosScanPt);
       fprintf(fpOut,"\n iCurrPosScanPt = %d\n",iCurrPosScanPt);

}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          ScanPtInsideImSqr
Input Parameters:  Null
Output Parameters: Null
Description:       Finds all the scan points which are included insided the imaginary square for 
                   each of the known obstacle
Last updates:      31st Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void ScanPtInsideImSqr(void)
{
     int iCounter, iTemp;
     
     for(iCounter=0;iCounter<NumOfKownObst;iCounter++)
     {
         for(iTemp=1;iTemp<=TotalNoOfScanPts;iTemp++)
         {
          if( (SimulationScanPtArr[iTemp].Source.X <= KnownObstacle[iCounter].ImgSqrNE.X) &&  // chk if scanPt within X direction limits
                   (SimulationScanPtArr[iTemp].Source.X >= KnownObstacle[iCounter].ImgSqrNW.X) &&
                   (SimulationScanPtArr[iTemp].Source.Y >= KnownObstacle[iCounter].ImgSqrNW.Y)&& // chk if scanPt within Y direction limits
                   (SimulationScanPtArr[iTemp].Source.Y <= KnownObstacle[iCounter].ImgSqrSW.Y)) 
          
                    {
                          SimulationScanPtArr[iTemp].IsScanPointReachable = 0; 
                          ScanPtArr[iTemp].IsScanPointReachable = 0;    //This is Very Important Statement which helps in evaluating logic for LocalPath                                
                          ScanPtArr[iTemp].ObstacleNo = iCounter;

                    }
         }                              

     }
     ScanPointReachabilityCheck();
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          CreateAltGoalPt
Input Parameters:  Null
Output Parameters: Null
Description:       create alterante goal points as per known obstacles
Last updates:      31st Dec 08 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void CreateAltGoalPt(void)
{
     int iCounter;
     int iTemp;
     int GoalPtNum;
     
     for(iCounter=1;iCounter<TotalNoOfScanPts;iCounter++)
     {
          if (ScanPtArr[iCounter].IsScanPointReachable == 0)
             continue; //if the scan pt itself not reachable dont update its alternate goal pts.
          GoalPtNum = ScanPtArr[iCounter].GoalPtNum;  
          if (ScanPtArr[GoalPtNum].IsScanPointReachable == 0)
          {
              for(iTemp=GoalPtNum;iTemp<TotalNoOfScanPts;iTemp++)
              {
                 if (ScanPtArr[ScanPtArr[iTemp].GoalPtNum].IsScanPointReachable == 1)
                 {
                    ScanPtArr[iCounter].AltGoalPtNum =  ScanPtArr[iTemp].GoalPtNum;
                    
                    break;                                                                
                 }//end-if
              }//end-for
          }//end-if                                           
     }//end-for
}


/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          UpdateAltGoalPt
Input Parameters:  iCurrPos - Current Scan Point Number
Output Parameters: Null
Description:       creates alterante goal point for curren scan point
Last updates:      2nd April 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void UpdateAltGoalPt(int iCurrPos)
{
     int iCounter;
     int iTemp;
     int GoalPtNum;
     
     for(iCounter=iCurrPos;iCounter<TotalNoOfScanPts;iCounter++)
     {
              GoalPtNum = ScanPtArr[iCounter].GoalPtNum; 
               if ((ScanPtArr[GoalPtNum].IsScanPointReachable == 1) && (ScanPtArr[GoalPtNum].Visited == 0))
                 {
                    ScanPtArr[iCurrPos].AltGoalPtNum =  GoalPtNum;
                    break;                                                                
                 }//end-if
                     
     }//end-for
     fprintf(fpOut,"\nScanPtArr[%d].GoalPtNum = %d\n",iCurrPos,ScanPtArr[iCurrPos].GoalPtNum);
     fprintf(fpOut,"\nScanPtArr[%d].AltGoalPtNum = %d\n",iCurrPos,ScanPtArr[iCurrPos].AltGoalPtNum);
     
     FieldCompletlyCovered = 1;
     for(iCounter=1;iCounter<=TotalNoOfScanPts;iCounter++)
     {
          if ((ScanPtArr[iCounter].Visited == 0) && (ScanPtArr[iCounter].IsScanPointReachable == 1))
          {
             FieldCompletlyCovered = 0; 
             break;                             
          } 
     }
     
}


/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          LocalPathNav
Input Parameters:  Source and goal scan point numbers
Output Parameters: Null
Description:       Navigates the vehicle around the obstacle to reach back to its normal path
                   For this its calculates intermediate goal points around the obstacle and 
                   hence creates a local path.
Last updates:      2nd Jan 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void LocalPathNav(int prevScanPtNum, int sourceScanPtNum, int goalScanPtNum, int Pattern)
{
     int localGoal;
     int localPrev;
     int NumofReachPtsofSource;
     int SrcReachPtNumber;
     int ObstPresent;
     int iCount,iTemp=0,iTemp2=0,iTemp3=0;
     char LocalGoalIsReachPtOfSource = 'N';
     int TemporaryInAccessible;
     int c;
    fprintf(fpOut,"\n\nInside LocalPathNav\n\n"); 
    fprintf(fpOut,"\nPattern = %d\n", Pattern); 
     if(LocalNavCounter==0) // only entries for main local path nav called from navigate field func
     {
         stackScanPtArray[LocalNavCounter].ScanPtNum = goalScanPtNum;
         mainPrev = prevScanPtNum;
         mainSource = sourceScanPtNum;
         mainTarget = goalScanPtNum;
     }

     LocalNavCounter ++; // count the number of times the local nav function is called in recurrsion
     //ScanPtArr[goalScanPtNum].Weightage = ScanPtArr[goalScanPtNum].Weightage + (LocalNavCounter/100);
     if (Pattern == 0)
        localGoal = FindOptimumLocalGoalPt(sourceScanPtNum,goalScanPtNum); //Find distance of each reach point of the alternate goal point from source scan point
     else
        localGoal = FindRightOptimumLocalGoalPt(sourceScanPtNum,goalScanPtNum); //Find distance of each reach point of the alternate goal point from source scan point
     fprintf(fpOut,"\n\n localGoal = %d\nDirection = %d",localGoal,OffPathDirection);

     fprintf(fpOut,"\nLocalNavCounter = %d\n",LocalNavCounter);

     stackScanPtArray[LocalNavCounter].ScanPtNum = localGoal; 
     stackScanPtArray[LocalNavCounter].Direction = OffPathDirection;

     //if Loop Condition arises, See The point which appears again in the loop
     //The direction taken at that point seems not to be correct.
     fprintf(fpOut,"\nCheck for Loop Condition for BestReachPt %d\n",localGoal);
     for (iTemp = 0; iTemp<LocalNavCounter;iTemp++)
     {
         fprintf(fpOut,"\n\nstackScanPtArray[%d]= %d\n",iTemp,stackScanPtArray[iTemp]);
         if ((stackScanPtArray[iTemp].ScanPtNum ==  localGoal) && (localGoal!=goalScanPtNum))
           {
              iTemp2 = iTemp;
              TemporaryInAccessible = stackScanPtArray[iTemp+1].ScanPtNum;
              TemporaryScanPtAccess[TemporaryScanPtAccessCtr++] = TemporaryInAccessible;

              fprintf(fpOut,"\n\Make %d ScanPt not reachable temporarily for this local path\n",TemporaryInAccessible);
              LocalNavCounter = iTemp2;
              break;
           }

     }      
 
    
     NumofReachPtsofSource = ScanPtArr[sourceScanPtNum].NumOfReachPts;
     for(iCount=0;iCount<=3;iCount++)
     {
 
         if ((ScanPtArr[sourceScanPtNum].ScanPtNumsOfReachPts[iCount] == localGoal))
         {
            LocalGoalIsReachPtOfSource = 'Y';
            break;
         }
        
     }//end-for


    if(mainSource == localGoal) //If possible use     //if (localGoal == mainTarget) condition

    {
          LocalNavCounter--;
          stackScanPtArray[LocalNavCounter+1].ScanPtNum = mainSource;
          stackScanPtArray[LocalNavCounter+2].ScanPtNum = mainPrev;                   
          stackCounter = LocalNavCounter+2;
          fputs("Move Vehicle in LoclPath Begin:\n",fpOut); 
          fprintf(fpOut,"\n stackCounter = %d\n",stackCounter);
          for(iTemp=0;iTemp<=stackCounter;iTemp++)          
          {    
               fprintf(fpOut,"\n istackScanPtArray[%d].ScanPtNum = %d\n\n",iTemp,stackScanPtArray[iTemp].ScanPtNum);

               for (iTemp2=stackCounter-1;iTemp2>iTemp;iTemp2--)
               {
                   if (stackScanPtArray[iTemp].ScanPtNum==stackScanPtArray[iTemp2].ScanPtNum)
                   {
                      fprintf(fpOut,"\n\nRemove All Entries between %d and %d\n",iTemp, iTemp2+1);
                      break;
                      
                   }
               }
               for (iTemp3 = iTemp+1;iTemp3<=iTemp2; iTemp3++)
               {
                  stackScanPtArray[iTemp3].ScanPtNum = -99;//stackScanPtArray[iTemp]; 
               }
               
          }//end-for
          //Following Block of code copies all valid values from stackScanPtArray to LocalPathScanPtArray
          iTemp = 0;
          LocalPathCounter = 0;
          while (iTemp <=stackCounter)
          {
             if (stackScanPtArray[iTemp].ScanPtNum !=-99)
             {
                LocalPathScanPtArray[LocalPathCounter] = stackScanPtArray[iTemp].ScanPtNum;
                LocalPathCounter++;
                                         
             }
             iTemp++;
          }
              fputs("Move Vehicle in LoclPath end:\n",fpOut); 
          
    }     
    else  
    {
        LocalPathNav(prevScanPtNum, sourceScanPtNum, localGoal,Pattern);
    }

}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          FindOptimumLocalGoalPt
Input Parameters:  source and alternate goal point numbers
Output Parameters: most optimum alternate goal point considering local path from left hand of the obstacle
Description:       Using following points the most optimum alternate goal point is selected 
                   -> if not the source scan point itself 
                   -> ifalready visited
                   -> if reachable

Last updates:      13th March 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
int FindOptimumLocalGoalPt(int SourcePt, int AltGoalPt)
{
    int NumOfReachPtsOfAltGoal,ReachPtNumber;
     int iCount,;
     int BestReachPt;
     int SourceX,SourceY,AltGoalX,AltGoalY,TargetX,TargetY;
     int iTemp=0,iTemp2=0;
     int ReachPt;
     int TemporaryInAccessible;
     
     fprintf(fpOut,"\nInside FindOptimumLocalGoalPt\n");  
     
     NumOfReachPtsOfAltGoal = ScanPtArr[AltGoalPt].NumOfReachPts;
     
     SourceY =  ScanPtArr[SourcePt].Source.Y; 
     SourceX =  ScanPtArr[SourcePt].Source.X;
     AltGoalY =  ScanPtArr[AltGoalPt].Source.Y; 
     AltGoalX =  ScanPtArr[AltGoalPt].Source.X;        
     TargetY =  ScanPtArr[mainTarget].Source.Y; 
     TargetX =  ScanPtArr[mainTarget].Source.X;         
           
             
     if(SourceY < TargetY)//--------------MAIN CASE 1------------------------------
     {
        if((SourceY < AltGoalY)&& (SourceX == AltGoalX))//CASE 1.1
        {
                    fprintf(fpOut,"\nInside Case 1.1\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on left
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on BOTTOM
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                        //if BOTTOM reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on right
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if right reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR; // if none of the reach points are accessible
                                return(AltGoalPt);//return the same alt goal with offpathdir flag nullified
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.1
        else if((SourceY < AltGoalY)&& (SourceX > AltGoalX))//CASE 1.2
        {
             fprintf(fpOut,"\nInside Case 1.2\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on left
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR  
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on bottom
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if bottom reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on right
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if right reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.2
        else if((SourceY < AltGoalY)&& (SourceX < AltGoalX))//CASE 1.3
        {
             fprintf(fpOut,"\nInside Case 1.3\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case BOTTOM 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.3
        else if((SourceY == AltGoalY)&& (SourceX > AltGoalX))//CASE 1.4
        {
             fprintf(fpOut,"\nInside Case 1.4\n");
               //consider reach pt on right
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if right reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable  
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.4
        else if((SourceY == AltGoalY)&& (SourceX < AltGoalX))//CASE 1.5
        {
             fprintf(fpOut,"\nInside Case 1.5\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on BOTTOM
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if BOTTOM reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.5
        else if((SourceY > AltGoalY)&& (SourceX == AltGoalX))//CASE 1.6
        {
             fprintf(fpOut,"\nInside Case 1.6\n");
               //consider reach pt on bottom
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if bottom reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on top
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if top reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.6
        else if((SourceY > AltGoalY)&& (SourceX > AltGoalX))//CASE 1.7
        {
             fprintf(fpOut,"\nInside Case 1.7\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.7
        else if((SourceY > AltGoalY)&& (SourceX < AltGoalX))//CASE 1.8
        {
             fprintf(fpOut,"\nInside Case 1.8\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                    //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on top
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if top reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.8
        
     }//MAIN CASE 1
     else if(SourceY > TargetY)//--------------MAIN CASE 2 ------------------------------
     {
        if((SourceY < AltGoalY)&& (SourceX == AltGoalX))//CASE 2.1
        {
                    fprintf(fpOut,"\nInside Case 2.1\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable  AND chk if OffPathDirection is not LEFTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on BOTTOM
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if BOTTOM reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on LEFT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if LEFT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case LEFT 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.1
        else if((SourceY < AltGoalY)&& (SourceX > AltGoalX))//CASE 2.2
        {
             fprintf(fpOut,"\nInside Case 2.2\n");
               //consider reach pt on RIGHT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on BOTTOM
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if BOTTOM reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on TOP
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if TOP reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.2
        else if((SourceY < AltGoalY)&& (SourceX < AltGoalX))//CASE 2.3
        {
             fprintf(fpOut,"\nInside Case 2.3\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on left
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.3
        else if((SourceY == AltGoalY)&& (SourceX > AltGoalX))//CASE 2.4
        {
             fprintf(fpOut,"\nInside Case 2.4\n");
               //consider reach pt on right
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if right reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on bottom
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                    //if bottom reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on left
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if left reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.4
        else if((SourceY == AltGoalY)&& (SourceX < AltGoalX))//CASE 2.5
        {
             fprintf(fpOut,"\nInside Case 2.5\n");
               //consider reach pt on LEFT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on bottom
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if bottom reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.5
        else if((SourceY > AltGoalY)&& (SourceX == AltGoalX))//CASE 2.6
        {
             fprintf(fpOut,"\nInside Case 2.6\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.6  
        else if((SourceY > AltGoalY)&& (SourceX < AltGoalX))//CASE 2.7
        {
             fprintf(fpOut,"\nInside Case 2.7\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable  AND chk if OffPathDirection is not LEFTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on LEFT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if LEFT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case LEFT 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.7
        else if((SourceY > AltGoalY)&& (SourceX > AltGoalX))//CASE 2.8
        {
                fprintf(fpOut,"\nInside Case 2.8\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.8
                
     }//MAIN CASE 2 and 3
     else if(SourceY == TargetY)//--------------MAIN CASE 3 ------------------------------
     {
        if((SourceY < AltGoalY)&& (SourceX == AltGoalX))//CASE 3.1
        {
                    fprintf(fpOut,"\nInside Case 3.1\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.1
        else if((SourceY < AltGoalY)&& (SourceX > AltGoalX))//CASE 3.2
        {
             fprintf(fpOut,"\nInside Case 3.2\n");
               //consider reach pt on RIGHT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on BOTTOM
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if BOTTOM reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on TOP
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if TOP reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.2
        else if((SourceY < AltGoalY)&& (SourceX < AltGoalX))//CASE 3.3
        {
             fprintf(fpOut,"\nInside Case 3.3\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on left
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.3
        else if((SourceY == AltGoalY)&& (SourceX > AltGoalX))//CASE 3.4
        {
             fprintf(fpOut,"\nInside Case 3.4\n");
               //consider reach pt on right
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if right reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on bottom
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                    //if bottom reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on left
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if left reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.4
        else if((SourceY == AltGoalY)&& (SourceX < AltGoalX))//CASE 3.5
        {
             fprintf(fpOut,"\nInside Case 3.5\n");
               //consider reach pt on LEFT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on bottom
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if bottom reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.5
        else if((SourceY > AltGoalY)&& (SourceX == AltGoalX))//CASE 3.6
        {
             fprintf(fpOut,"\nInside Case 3.6\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.6  
        else if((SourceY > AltGoalY)&& (SourceX < AltGoalX))//CASE 3.7
        {
             fprintf(fpOut,"\nInside Case 3.7\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.7
        else if((SourceY > AltGoalY)&& (SourceX > AltGoalX))//CASE 3.8
        {
                fprintf(fpOut,"\nInside Case 3.8\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.8
                
     }//MAIN CASE 3
         return(BestReachPt);

}//FindDistOfReachPts

int CheckIfTemporaryInAccessible(int ScanPt)   
{   
    int iReturn = ScanPt;
    int iTemp;
          for(iTemp=0;iTemp<TemporaryScanPtAccessCtr;iTemp++)     
          {
            if (TemporaryScanPtAccess[iTemp] == ScanPt)
            {   
               iReturn = -99;
               break;
            }//end-if
            
          } //end-for 
    return (iReturn);
}   

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          FindRightOptimumLocalGoalPt
Input Parameters:  source and alternate goal point numbers
Output Parameters: most optimum alternate goal point considering local path from right hand of the obstacle
Description:       Using following points the most optimum alternate goal point is selected 
                   -> if not the source scan point itself 
                   -> ifalready visited
                   -> if reachable

Last updates:      13th March 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
int FindRightOptimumLocalGoalPt(int SourcePt, int AltGoalPt)
{
    int NumOfReachPtsOfAltGoal,ReachPtNumber;
     int iCount;
     int BestReachPt;
     int SourceX,SourceY,AltGoalX,AltGoalY,TargetX,TargetY;
     int iTemp=0,iTemp2=0;
     int ReachPt;
     int TemporaryInAccessible;
     
     
     NumOfReachPtsOfAltGoal = ScanPtArr[AltGoalPt].NumOfReachPts;
     
     SourceY =  ScanPtArr[SourcePt].Source.Y; 
     SourceX =  ScanPtArr[SourcePt].Source.X;
     AltGoalY =  ScanPtArr[AltGoalPt].Source.Y; 
     AltGoalX =  ScanPtArr[AltGoalPt].Source.X;        
     TargetY =  ScanPtArr[mainTarget].Source.Y; 
     TargetX =  ScanPtArr[mainTarget].Source.X;         
           
             
     if(SourceY < TargetY)//--------------MAIN CASE 1------------------------------
     {
        if((SourceY < AltGoalY)&& (SourceX == AltGoalX))//CASE 1.1
        {
                    fprintf(fpOut,"\nInside Case 1.1\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on right
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if right reachpt is reachable  AND chk if OffPathDirection is not LEFTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on BOTTOM
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                        //if BOTTOM reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on left
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if left reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case left 
                            else
                            {
                                OffPathDirection = NULLDIR; // if none of the reach points are accessible
                                return(AltGoalPt);//return the same alt goal with offpathdir flag nullified
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.1
        else if((SourceY < AltGoalY)&& (SourceX > AltGoalX))//CASE 1.2
        {
             fprintf(fpOut,"\nInside Case 1.2\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on left
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR  
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on bottom
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if bottom reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on right
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if right reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.2
        else if((SourceY < AltGoalY)&& (SourceX < AltGoalX))//CASE 1.3
        {
             fprintf(fpOut,"\nInside Case 1.3\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on BOTTOM
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if BOTTOM reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on LEFT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if LEFT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case LEFT 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.3
        else if((SourceY == AltGoalY)&& (SourceX > AltGoalX))//CASE 1.4
        {
             fprintf(fpOut,"\nInside Case 1.4\n");
               //consider reach pt on right
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if right reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable  
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.4
        else if((SourceY == AltGoalY)&& (SourceX < AltGoalX))//CASE 1.5
        {
             fprintf(fpOut,"\nInside Case 1.5\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.5
        else if((SourceY > AltGoalY)&& (SourceX == AltGoalX))//CASE 1.6
        {
             fprintf(fpOut,"\nInside Case 1.6\n");
               //consider reach pt on bottom
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if bottom reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on top
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if top reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.6
        else if((SourceY > AltGoalY)&& (SourceX > AltGoalX))//CASE 1.7
        {
             fprintf(fpOut,"\nInside Case 1.7\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.7
        else if((SourceY > AltGoalY)&& (SourceX < AltGoalX))//CASE 1.8
        {
             fprintf(fpOut,"\nInside Case 1.8\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                    //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 1.8
        
     }//MAIN CASE 1
     else if(SourceY > TargetY)//--------------MAIN CASE 2 ------------------------------
     {
        if((SourceY < AltGoalY)&& (SourceX == AltGoalX))//CASE 2.1
        {
                    fprintf(fpOut,"\nInside Case 2.1\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.1
        else if((SourceY < AltGoalY)&& (SourceX > AltGoalX))//CASE 2.2
        {
             fprintf(fpOut,"\nInside Case 2.2\n");
               //consider reach pt on RIGHT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on BOTTOM
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if BOTTOM reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on TOP
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if TOP reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.2
        else if((SourceY < AltGoalY)&& (SourceX < AltGoalX))//CASE 2.3
        {
             fprintf(fpOut,"\nInside Case 2.3\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on BOTTOM
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if BOTTOM reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.3
        else if((SourceY == AltGoalY)&& (SourceX > AltGoalX))//CASE 2.4
        {
             fprintf(fpOut,"\nInside Case 2.4\n");
               //consider reach pt on right
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if right reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on bottom
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                    //if bottom reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on left
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if left reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.4
        else if((SourceY == AltGoalY)&& (SourceX < AltGoalX))//CASE 2.5
        {
             fprintf(fpOut,"\nInside Case 2.5\n");
               //consider reach pt on LEFT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on BOTTOM
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if BOTTOM reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on TOP
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if TOP reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case TOP 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.5
        else if((SourceY > AltGoalY)&& (SourceX == AltGoalX))//CASE 2.6
        {
             fprintf(fpOut,"\nInside Case 2.6\n");
               //consider reach pt on RIGHT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.6  
        else if((SourceY > AltGoalY)&& (SourceX < AltGoalX))//CASE 2.7
        {
             fprintf(fpOut,"\nInside Case 2.7\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on RIGHT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if RIGHT reachpt is reachable  AND chk if OffPathDirection is not LEFTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = RIGHTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on LEFT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if LEFT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case LEFT 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.7
        else if((SourceY > AltGoalY)&& (SourceX > AltGoalX))//CASE 2.8
        {
                fprintf(fpOut,"\nInside Case 2.8\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 2.8
                
     }//MAIN CASE 2 and 3
     else if(SourceY == TargetY)//--------------MAIN CASE 3 ------------------------------
     {
        if((SourceY < AltGoalY)&& (SourceX == AltGoalX))//CASE 3.1
        {
                    fprintf(fpOut,"\nInside Case 3.1\n");
               //consider reach pt on top
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if top reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.1
        else if((SourceY < AltGoalY)&& (SourceX > AltGoalX))//CASE 3.2
        {
             fprintf(fpOut,"\nInside Case 3.2\n");
               //consider reach pt on RIGHT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if RIGHT reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on BOTTOM
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if BOTTOM reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on LEFT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if LEFT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = LEFTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on TOP
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if TOP reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = TOPDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.2
        else if((SourceY < AltGoalY)&& (SourceX < AltGoalX))//CASE 3.3
        {
             fprintf(fpOut,"\nInside Case 3.3\n");
               //consider reach pt on TOP
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if TOP reachpt is reachable AND chk if OffPathDirection is not BOTTOMDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = TOPDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on left
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if left reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on RIGHT
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if RIGHT reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on BOTTOM
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if BOTTOM reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.3
        else if((SourceY == AltGoalY)&& (SourceX > AltGoalX))//CASE 3.4
        {
             fprintf(fpOut,"\nInside Case 3.4\n");
               //consider reach pt on right
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if right reachpt is reachable AND chk if OffPathDirection is not LEFTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = RIGHTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on bottom
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                    //if bottom reachpt is reachable  AND chk if OffPathDirection is not TOPDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = BOTTOMDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on left
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if left reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = LEFTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.4
        else if((SourceY == AltGoalY)&& (SourceX < AltGoalX))//CASE 3.5
        {
             fprintf(fpOut,"\nInside Case 3.5\n");
               //consider reach pt on LEFT
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if LEFT reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on bottom
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if bottom reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = BOTTOMDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.5
        else if((SourceY > AltGoalY)&& (SourceX == AltGoalX))//CASE 3.6
        {
             fprintf(fpOut,"\nInside Case 3.6\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt); 
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.6  
        else if((SourceY > AltGoalY)&& (SourceX < AltGoalX))//CASE 3.7
        {
             fprintf(fpOut,"\nInside Case 3.7\n");
               //consider reach pt on left
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if left reachpt is reachable AND chk if OffPathDirection is not RIGHTDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = LEFTDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on top
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if top reachpt is reachable  AND chk if OffPathDirection is not BOTTOMDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = TOPDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on right
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if right reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = RIGHTDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on bottom
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if bottom reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = BOTTOMDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.7
        else if((SourceY > AltGoalY)&& (SourceX > AltGoalX))//CASE 3.8
        {
                fprintf(fpOut,"\nInside Case 3.8\n");
               //consider reach pt on BOTTOM
               ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[3];
               ReachPt = CheckIfTemporaryInAccessible(ReachPt);
               //if BOTTOM reachpt is reachable AND chk if OffPathDirection is not TOPDIR
               if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != TOPDIR))
               {
                   BestReachPt = ReachPt;
                   OffPathDirection = BOTTOMDIR;
                   return(BestReachPt);
               }            
               else
               {    //consider reach pt on LEFT
                    ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[0]; 
                    ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                    //if LEFT reachpt is reachable  AND chk if OffPathDirection is not RIGHTDIR                   
                    if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != RIGHTDIR))
                   {
                       BestReachPt = ReachPt;
                       OffPathDirection = LEFTDIR;
                       return(BestReachPt);
                   }
                   else
                   {//consider reach pt on TOP
                        ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[2]; 
                        ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                        //if TOP reachpt is reachable
                        if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != BOTTOMDIR))
                        {
                           BestReachPt = ReachPt;
                           OffPathDirection = TOPDIR;
                           return(BestReachPt);
                        }  
                        else
                        {//consider reach pt on RIGHT
                            ReachPt = ScanPtArr[AltGoalPt].ScanPtNumsOfReachPts[1]; 
                            ReachPt = CheckIfTemporaryInAccessible(ReachPt);
                            //if RIGHT reachpt is reachable
                            if((ReachPt != -99)&& (ScanPtArr[ReachPt].IsScanPointReachable == 1)&&(OffPathDirection != LEFTDIR))
                            {
                               BestReachPt = ReachPt;
                               OffPathDirection = RIGHTDIR;
                               return(BestReachPt);
                            }//end case right 
                            else
                            {
                                OffPathDirection = NULLDIR;
                                return(AltGoalPt);
                            }
                        }//end case bottom  
                   }//end case left
               }///end case top
        }////CASE 3.8
                
     }//MAIN CASE 3
         return(BestReachPt);

}//FindDistOfReachPts

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          GetScanPtNum
Input Parameters:  X and Y Co-ordinates
Output Parameters: ScanPtNum of Scan point whose co-ordinates match input X and Y Co-ordinates
Description:       Finds the Scan point whose co ordinates match the input parameters and returns scan point number
Last updates:      2nd Jan 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

int GetScanPtNum(int X, int Y)
{
     int iCounter;
     int ScanPtNum = 0;
     
     for(iCounter=1;iCounter<=TotalNoOfScanPts;iCounter++)
     {
          if ((ScanPtArr[iCounter].Source.X == X) && (ScanPtArr[iCounter].Source.Y == Y))
          {
             ScanPtNum = ScanPtArr[iCounter].ScanPtNum;
             break;
          }
          
     }
    return (ScanPtNum); 
     
}
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          ReadInputLineFromFile
Input Parameters:  Null
Output Parameters: Null
Description:       Reads all input parameters(vehicle width, field length, etc) from the 
                   input txt file. Also data related to known obstacle is read from this
                   file and the imajinary sqr corner coordinates are calculated
Last updates:      15th Jan 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void ReadInputLineFromFile(void)
{
     char *sValue = NULL; //variable used in strtok function to split string
     char LineInNetList[150], LineWOComment[150]; //string variables used in string manipulation 
     int iCounter = 0; //variable used for checking status of input line.  
     int Ctr =0, iCount, sVar;
     int north,south,west,east;
     char inputfromuser;
     FILE *fp;				//Declares file pointer fp of type FILE     
     fp = fopen("Input.txt", "r"); //opens NetList.txt in read mode. File pointer fp points to this file.
     //Check if file specified can be opened or not. If file can not be opened, exit this function with return value 0.
     if (fp==NULL) 
     {
        printf("File Input.txt can not be opened\n");
        inputfromuser = getc(stdin);
        return;
     }       
     //following line reads a line from netlist in variable LineInNetList.
     for (Ctr = 1; Ctr <=5; Ctr++)
     {
         fgets(LineInNetList,150,fp);
         //if eof is reached, pass control out of the function
         if(feof(fp)) return;
         //following while loop parse input string and copies data before *(comment) in string 'LineWOComment'
         iCount = 0;
         while ((LineInNetList[iCount] !='\0') && (LineInNetList[iCount] !='*') && (LineInNetList[iCount] !='\n'))
               LineWOComment[iCount] = LineInNetList[iCount++];
         LineWOComment[iCount]='\0';  //terminate 'LineWOComment' string with null character.
         sValue = strtok(LineWOComment, " \t\n");
         sVar = atoi(sValue);
          if (Ctr ==1)
             FieldLength = sVar;
          else if (Ctr ==2)
             FieldWidth = sVar;
          else if (Ctr ==3)
             LidarRange = sVar;         
          else if (Ctr ==4)
             VehicleWidth = sVar;         
          else if (Ctr ==5)
             NumOfKownObst = sVar; 
     }//end-for

     for (Ctr = 1; Ctr <=NumOfKownObst; Ctr++)
     {
         //left co-ordinates
         fgets(LineInNetList,150,fp);
         //following while loop parse input string and copies data before *(comment) in string 'LineWOComment'
         iCount = 0;
         while ((LineInNetList[iCount] !='\0') && (LineInNetList[iCount] !='*') && (LineInNetList[iCount] !='\n'))
               LineWOComment[iCount] = LineInNetList[iCount++];
         LineWOComment[iCount]='\0';  //terminate 'LineWOComment' string with null character.
         sValue = strtok(LineWOComment, " \t\n");
         KnownObstacle[Ctr-1].west = atoi(sValue);

         //northmost   co-ordinates
         fgets(LineInNetList,150,fp);

         //following while loop parse input string and copies data before *(comment) in string 'LineWOComment'
         iCount = 0;
         while ((LineInNetList[iCount] !='\0') && (LineInNetList[iCount] !='*') && (LineInNetList[iCount] !='\n'))
               LineWOComment[iCount] = LineInNetList[iCount++];
         LineWOComment[iCount]='\0';  //terminate 'LineWOComment' string with null character.
         sValue = strtok(LineWOComment, " \t\n");
         KnownObstacle[Ctr-1].north = atoi(sValue);

         //rightmost  co-ordinates
         fgets(LineInNetList,150,fp);

         //following while loop parse input string and copies data before *(comment) in string 'LineWOComment'
         iCount = 0;
         while ((LineInNetList[iCount] !='\0') && (LineInNetList[iCount] !='*') && (LineInNetList[iCount] !='\n'))
               LineWOComment[iCount] = LineInNetList[iCount++];
         LineWOComment[iCount]='\0';  //terminate 'LineWOComment' string with null character.
         sValue = strtok(LineWOComment, " \t\n");
         KnownObstacle[Ctr-1].east = atoi(sValue);

         //bottommost  co-ordinates
         fgets(LineInNetList,150,fp);
    
         //following while loop parse input string and copies data before *(comment) in string 'LineWOComment'
         iCount = 0;
         while ((LineInNetList[iCount] !='\0') && (LineInNetList[iCount] !='*') && (LineInNetList[iCount] !='\n'))
               LineWOComment[iCount] = LineInNetList[iCount++];
         LineWOComment[iCount]='\0';  //terminate 'LineWOComment' string with null character.
         sValue = strtok(LineWOComment, " \t\n");
         KnownObstacle[Ctr-1].south = atoi(sValue);                  

          //extreme limits of imaginary square around the known obstacle
          north = KnownObstacle[Ctr-1].north - (VehicleWidth/2);
          south = KnownObstacle[Ctr-1].south + (VehicleWidth/2);
          west =   KnownObstacle[Ctr-1].west - (VehicleWidth/2);
          east = (VehicleWidth/2) + KnownObstacle[Ctr-1].east;
          
          //corner points of the imginary square
          KnownObstacle[Ctr-1].ImgSqrNE.X = east;    // upper left
          KnownObstacle[Ctr-1].ImgSqrNE.Y = north;      // corner
          KnownObstacle[Ctr-1].ImgSqrNW.X = west;    // upper right
          KnownObstacle[Ctr-1].ImgSqrNW.Y = north;      // corner
          KnownObstacle[Ctr-1].ImgSqrSE.X = east;     // lower left
          KnownObstacle[Ctr-1].ImgSqrSE.Y = south;   // corner
          KnownObstacle[Ctr-1].ImgSqrSW.X = west;    // lower right
          KnownObstacle[Ctr-1].ImgSqrSW.Y = south;   // corner  
                                               
     }//end-for

}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          CreateImgSqrForUnkownObst
Input Parameters:  Number of obstacles, Extreme North, South, East, West points of each known obstacle
Output Parameters: Null
Description:       Using the data of unknown obstacles this routine creates a virtual square 
                   around the obstacles. Also it finds out all the scan points which 
                   lie inside the imaginary sqauare
Last updates:      25th Jan 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 
void CreateImgSqrForUnkownObst(void)
{
      
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrNE.X = UnkownImagSqr_Right_X;    // upper left
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrNE.Y = UnkownImagSqr_Top_Y;      // corner
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrNW.X = UnkownImagSqr_Left_X;    // upper right
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrNW.Y = UnkownImagSqr_Top_Y;      // corner
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrSE.X = UnkownImagSqr_Right_X;     // lower left
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrSE.Y = UnkownImagSqr_Bottom_Y;   // corner
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrSW.X = UnkownImagSqr_Left_X;    // lower right
          UnKnownObstacle[NumOfUnKownObst-1].ImgSqrSW.Y = UnkownImagSqr_Bottom_Y;   // corner  
          
      
          
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          ScanPtInsideUnkownImSqr
Input Parameters:  Null
Output Parameters: Null
Description:       Finds all the scan points which are included insided the imaginary 
                   square for each of the unknown obstacle
Last updates:      20th Jan 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void ScanPtInsideUnkownImSqr(void)
{
     int temp;
     char ScanPtNo;
 
     int iCounter, iTemp;
     
     for(iCounter=0;iCounter<NumOfUnKownObst;iCounter++)
     {
             for(iTemp=1;iTemp<=TotalNoOfScanPts;iTemp++)
             {
             if( (SimulationScanPtArr[iTemp].Source.X <= UnKnownObstacle[iCounter].ImgSqrNE.X) &&  // chk if scanPt within X direction limits
                 (SimulationScanPtArr[iTemp].Source.X >= UnKnownObstacle[iCounter].ImgSqrNW.X) &&
                 (SimulationScanPtArr[iTemp].Source.Y >= UnKnownObstacle[iCounter].ImgSqrNW.Y)&& // chk if scanPt within Y direction limits
                 (SimulationScanPtArr[iTemp].Source.Y <= UnKnownObstacle[iCounter].ImgSqrSW.Y)) 
              
                       {
                              SimulationScanPtArr[iTemp].IsScanPointReachable = 0; 
                              ScanPtArr[iTemp].IsScanPointReachable = 0;    //This is Very Important Statement which helps in evaluating logic for LocalPath                                
    
                        }
             } //end-for   

         } //end-for
       ScanPointReachabilityCheck();
       
}//end-void UnKnownImagSqrDetection(void)


/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          ScanPointReachabilityCheck
Input Parameters:  Null
Output Parameters: Null
Description:       Makes the Scan Point Not Reachable when All It's Reach points are 
                   not reachable
                   This is called from UpdateCar after local path navigation completed.
Last updates:      20th Jan 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void ScanPointReachabilityCheck(void)
{
     int iTemp;
     int iCount;
     int isScanPtReachable = 0;
     int ReachPt = 0;
     int NumOfReachPtsInAccessible = 0;
     for(iTemp=1;iTemp<=TotalNoOfScanPts;iTemp++)
     {
           isScanPtReachable = 0;
           for(iCount=0;iCount<4;iCount++) 
           {
              ReachPt = ScanPtArr[iTemp].ScanPtNumsOfReachPts[iCount];
              if (ReachPt == -99)
                 continue;

              if (ScanPtArr[ReachPt].IsScanPointReachable==1)
              {
                 isScanPtReachable=1;
              }
             
           }//end-for (iCcount...
           //fprintf(fpOut,"\n\n isScanPtReachable = %d\n\n",isScanPtReachable);
           if (ScanPtArr[iTemp].IsScanPointReachable==1 && isScanPtReachable==0)
           {
              ScanPtArr[iTemp].IsScanPointReachable = 0; 
           }
           fprintf(fpOut,"\n\n ScanPtArr[%d]=%d IsReachable %d\n",iTemp,ScanPtArr[iTemp].ScanPtNum,ScanPtArr[iTemp].IsScanPointReachable);
           
     }//end-for
     
     //Following For-Loop checks if Last ScanPoint is reachable. 
     //If It's not, it looks for another ScanPoint that is reachable and has maximum number for ScanPt
     for(iTemp=TotalNoOfScanPts;iTemp>0;iTemp--)     
     {
        if (ScanPtArr[iTemp].IsScanPointReachable==1)
           break;   
     }
     TotalNoOfScanPts = iTemp;
     
     

}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          LocalPathNavReInitialize
Input Parameters:  Null
Output Parameters: Null
Description:       Following function is added to ReInitialize variables needed in computing 
                   local path
                   This is called from UpdateCar after local path navigation completed.
Last updates:      2nd Feb 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void LocalPathNavReInitialize(void)
{          
           int iTemp;
           LocalPathPrevScanPt = stackScanPtArray[1].ScanPtNum;          
           LocalNavCounter = 0;
           mainSource =0;
           mainPrev =0;
           OffPathDirection = NULLDIR;  
          
          for(iTemp=0;iTemp<=stackCounter;iTemp++)                                                      
          {                                                                                            
             stackScanPtArray[iTemp].ScanPtNum = 0;
          }
          stackCounter = 0;  
          for(iTemp=0;iTemp<TemporaryScanPtAccessCtr;iTemp++)     
          {
            TemporaryScanPtAccess[iTemp]=0;
          }
          TemporaryScanPtAccessCtr = 0;
}                 
  
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          swap
Input Parameters:  Pointers to two numbers to be swaped
Output Parameters: Swaped numbers
Description:       Swap the given two numbers
Last updates:      15nd March 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void swap(int *x,int *y)   
{   
   int temp;   
   temp = *x;   
   *x = *y;   
   *y = temp;   
}   

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          bublesort
Input Parameters:  The Array of Unknown_X_Arr and the Unknown_X_Arr used for storing all the points
                   of the unknown free line
Output Parameters: assending sorted array
Description:       Sorts the Unknown_X_Arr and Unknown_Y_Arr in an assending manner
Last updates:      15nd March 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void bublesort(int list1[],int n)   
{   
   int i,j;   
   for(i=0;i<(n-1);i++)   
      for(j=0;j<(n-(i+1));j++)   
             if(list1[j] > list1[j+1])  
             { 
                    swap(&list1[j],&list1[j+1]);   //Unknown_X_Arr
                    
             }
}  

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Function:          ArrayCopy
Input Parameters:  
Output Parameters: dest array with values
Description:       Copies array values from one array to another
Last updates:      15nd March 09
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */ 

void ArrayCopy(int list1[],int list2[],int n)   
{   
   int i;
   for(i=0;i<n;i++)   
      list2[i] = list1[i];

}  

void OptimumPathDecision(void)
{
     fprintf(fpOut,"\nInside OptimumPathDecision\n"); 
     int iCounter, iTemp;
     int iLeftCounter = 0;
     int iRightCounter = 0;
     
     
     LocalPathNav(iPrevScanPt, iCurrPosScanPt, ScanPtArr[iCurrPosScanPt].AltGoalPtNum, 0);
     ArrayCopy(LocalPathScanPtArray,LocalLeftPathScanPtArray,LocalPathCounter);
     iLeftCounter = LocalPathCounter;
     LocalPathCounter = 0;
     LocalNavCounter = 0;
     
     OffPathDirection = NULLDIR;
     LocalPathNav(iPrevScanPt, iCurrPosScanPt, ScanPtArr[iCurrPosScanPt].AltGoalPtNum, 1);
     ArrayCopy(LocalPathScanPtArray,LocalRightPathScanPtArray,LocalPathCounter);
     iRightCounter = LocalPathCounter;
     LocalPathCounter = 0;
     
     fprintf(fpOut,"\niLeftCounter = %d\niRightCounter = %d\n",iLeftCounter,iRightCounter); 
     
     if (iLeftCounter<=iRightCounter)
     {
     ArrayCopy(LocalLeftPathScanPtArray,LocalPathScanPtArray,iLeftCounter);
     LocalPathCounter = iLeftCounter;     
     }
     else
     {
     ArrayCopy(LocalRightPathScanPtArray,LocalPathScanPtArray,iLeftCounter);
     LocalPathCounter = iRightCounter;     
     }
     
     
     
}
